@@ -1,40 +1,48 @@
 #!/usr/bin/perl
+# $Id: bench.pl,v 1.9 2005/08/22 20:26:48 tina Exp $
 use strict;
 use warnings;
 use lib qw(blib/lib ../blib/lib);
+my $count = 0;
+my $ht_file = 'test.htc';
+#$ht_file = 'test.htc.20';
 my %use = (
 	'HTML::Template' => 0,
 	'HTML::Template::Compiled' => 0,
 	'HTML::Template::JIT' => 0,
 	'Template' => 0,
+	# not yet
+#	'Text::ScriptTemplate' => 0,
 );
 for my $key (keys %use) {
 	eval "require $key";
 	$use{$key} = 1 unless $@;
-    print "using $key " . ($use{$key}?"yes":"no")."\n";
+	my $version = $use{$key} ? $key->VERSION : "-";
+    printf "using %25s %s\n", $key, $version;
 }
+$HTML::Template::Compiled::NEW_CHECK = 10;
 use Benchmark;
-$HTML::Template::Compiled::NEW_CHECK = 30; # 30sec
 my $debug = 0;
 $ENV{'HTML_TEMPLATE_ROOT'} = "examples";
 sub new_htc {
 	my $t1 = HTML::Template::Compiled->new(
 		#path => 'examples',
-		#case_insensitive => 1, # slow down
+		#case_sensitive => 0, # slow down
 		loop_context_vars => 1,
-		filename => 'test.html',
+		filename => $ht_file,
 		debug => $debug,
 		# note that you have to create the cachedir
 		# first, otherwise it will run without cache
-		#cache_dir => "cache/htc",
+		cache_dir => "cache/htc",
 	);
 	return $t1;
 }
 sub new_ht {
 	my $t2 = HTML::Template->new(
+		# case_sensitive => 1,
 		loop_context_vars => 1,
 		#path => 'examples',
-		filename => 'test.html',
+		filename => $ht_file,
 		cache => 1,
 	);
 	return $t2;
@@ -43,7 +51,7 @@ sub new_htj {
 	my $t2 = HTML::Template::JIT->new(
 		loop_context_vars => 1,
 		#path => 'examples',
-		filename => 'test.html',
+		filename => $ht_file,
 		cache => 1,
 		jit_path => '/tmp/jit',
 	);
@@ -55,8 +63,13 @@ sub new_tt {
 		COMPILE_DIR => 'cache/tt',
 	);
 }
+
+sub new_st {
+	my $st = Text::ScriptTemplate->new;
+	$st->load("examples/template.st");
+}
 my %params = (
-	name => 'merlyn',
+	name => '',
 	loopa => [{a=>3},{a=>4},{a=>5}],
 	#a => [qw(b c d)],
 	loopb => [{ inner => 23 }],
@@ -75,6 +88,7 @@ my %params = (
 sub output {
 	my $t = shift;
 	return unless defined $t;
+	$params{name} = (ref $t).' '.$count++;
 	$t->param(%params);
 	#print $t->{code} if exists $t->{code};
 	my $out = $t->output;
@@ -93,33 +107,34 @@ sub output_tt {
 }
 
 my $gobal_htc = $use{'HTML::Template::Compiled'} ? new_htc : undef;
-my $gobal_ht = $use{'HTML::Template'} ? new_ht : undef;;
-my $gobal_htj = $use{'HTML::Template::JIT'} ? new_htj : undef;;
-my $gobal_tt = $use{'Template'} ? new_tt : undef;;
+my $gobal_ht = $use{'HTML::Template'} ? new_ht : undef;
+my $gobal_htj = $use{'HTML::Template::JIT'} ? new_htj : undef;
+my $gobal_tt = $use{'Template'} ? new_tt : undef;
 if(1) {
-timethese ($ARGV[0]||100, {
+timethese ($ARGV[0]||-1, {
 		$use{'HTML::Template::Compiled'} ? (
             # deactivate memory cache
             #new_htc_w_clear_cache => sub {my $t = new_htc();$t->clear_cache},
             # normal, with memory cache
-            new_htc => sub {my $t = new_htc()},
-            output_htc => sub {output($gobal_htc)},
+						#new_htc => sub {my $t = new_htc()},
+						#output_htc => sub {output($gobal_htc)},
+						all_htc => sub {my $t = new_htc();output($t)},
         ) : (),
 		$use{'HTML::Template'} ? (
-            new_ht => sub {my $t = new_ht()},
-            output_ht => sub {output($gobal_ht)},
+			#new_ht => sub {my $t = new_ht()},
+			#output_ht => sub {output($gobal_ht)},
+						all_ht => sub {my $t = new_ht();output($t)},
         ) : (),
         $use{'HTML::Template::JIT'} ? (
-            new_htj => sub {my $t = new_htj();},
-            output_htj => sub {output($gobal_htj)},
+					#new_htj => sub {my $t = new_htj();},
+					#output_htj => sub {output($gobal_htj)},
+						all_htj => sub {my $t = new_htj();output($t)},
         ) : (),
         $use{'Template'} ? (
-            new_tt => sub {my $t = new_tt();},
-            output_tt => sub {output_tt($gobal_tt)},
+					#new_tt => sub {my $t = new_tt();},
+					#output_tt => sub {output_tt($gobal_tt)},
+						all_tt => sub {my $t = new_tt();output_tt($t)},
         ): (),
 	});
 }
 __END__
-my $file = "templates/test.tt";
-my $filehtc = "test.htc";
-$tt->process($file, $vars,);
